@@ -74,6 +74,7 @@ class Bank(object):
                            BiomajConfig.global_config.get('GENERAL', 'db.name'))
 
         self.banks = MongoConnector.banks
+        self.history = MongoConnector.history
         self.bank = self.banks.find_one({'name': self.name})
 
         if self.bank is None:
@@ -107,6 +108,22 @@ class Bank(object):
         else:
             return False
 
+    @staticmethod
+    def get_history(limit=100):
+        """
+        Get list of bank update/remove operations
+        """
+        if MongoConnector.db is None:
+            MongoConnector(BiomajConfig.global_config.get('GENERAL', 'db.url'),
+                           BiomajConfig.global_config.get('GENERAL', 'db.name'))
+
+        hist_list = []
+        hist = MongoConnector.history.find({}).sort("start", -1).limit(limit)
+        for h in hist:
+            del h['_id']
+            hist_list.append(h)
+        return hist_list
+
     @staticmethod
     def get_banks_disk_usage():
         """
@@ -561,10 +578,18 @@ class Bank(object):
                           'freeze': False}
             self.bank['production'].append(production)
             self.banks.update({'name': self.name},
-                              {'$push': {'production': production},
-                               '$pull': {'pending': {'release': self.session.get('release'),
-                                                     'id': self.session._session['id']}}
-                               })
+                              {
+                              '$push': {'production': production}
+                              })
+            self.banks.update({'name': self.name},
+                              {
+                              '$pull': {
+                                  'pending': {
+                                      'release': self.session.get('release'),
+                                      'id': self.session._session['id']
+                                  }
+                              }
+                              })
 
         self.bank = self.banks.find_one({'name': self.name})
 
@@ -917,6 +942,8 @@ class Bank(object):
         :type force: bool
         :return: bool
         """
+        start_time = datetime.now()
+        start_time = time.mktime(start_time.timetuple())
         if not force:
             has_freeze = False
             for prod in self.bank['production']:
@@ -932,13 +959,35 @@ class Bank(object):
         bank_data_dir = self.get_data_dir()
         logging.warn('DELETE ' + bank_data_dir)
         if os.path.exists(bank_data_dir):
-            shutil.rmtree(bank_data_dir)
+            try:
+                shutil.rmtree(bank_data_dir)
+            except Exception:
+                logging.exception('Failed to delete bank directory: ' + bank_data_dir)
+                logging.error('Bank will be deleted but some files/dirs may still be present on system, you can safely manually delete them')
         bank_offline_dir = os.path.join(self.config.get('data.dir'), self.config.get('offline.dir.name'))
         if os.path.exists(bank_offline_dir):
-            shutil.rmtree(bank_offline_dir)
+            try:
+                shutil.rmtree(bank_offline_dir)
+            except Exception:
+                logging.exception('Failed to delete bank offline directory: ' + bank_offline_dir)
+                logging.error('Bank will be deleted but some files/dirs may still be present on system, you can safely manually delete them')
         bank_log_dir = os.path.join(self.config.get('log.dir'), self.name)
         if os.path.exists(bank_log_dir) and self.no_log:
-            shutil.rmtree(bank_log_dir)
+            try:
+                shutil.rmtree(bank_log_dir)
+            except Exception:
+                logging.exception('Failed to delete bank log directory: ' + bank_log_dir)
+                logging.error('Bank will be deleted but some files/dirs may still be present on system, you can safely manually delete them')
+        end_time = datetime.now()
+        end_time = time.mktime(end_time.timetuple())
+        self.history.insert({
+            'bank': self.name,
+            'error': False,
+            'start': start_time,
+            'end': end_time,
+            'action': 'remove',
+            'updated': None
+        })
         return True
 
     def get_status(self):
@@ -1012,6 +1061,8 @@ class Bank(object):
         """
         release = str(release)
         logging.warning('Bank:' + self.name + ':Remove')
+        start_time = datetime.now()
+        start_time = time.mktime(start_time.timetuple())
 
         if not self.is_owner():
             logging.error('Not authorized, bank owned by ' + self.bank['properties']['owner'])
@@ -1047,8 +1098,17 @@ class Bank(object):
         # Reset status, we take an update session
         res = self.start_remove(session)
         self.session.set('workflow_status', res)
-
         self.save_session()
+        end_time = datetime.now()
+        end_time = time.mktime(end_time.timetuple())
+        self.history.insert({
+            'bank': self.name,
+            'error': not res,
+            'start': start_time,
+            'end': end_time,
+            'action': 'remove',
+            'updated': None
+        })
 
         return res
 
@@ -1061,6 +1121,8 @@ class Bank(object):
         :return: bool
         """
         logging.warning('Bank:' + self.name + ':Update')
+        start_time = datetime.now()
+        start_time = time.mktime(start_time.timetuple())
 
         if not self.is_owner():
             logging.error('Not authorized, bank owned by ' + self.bank['properties']['owner'])
@@ -1108,7 +1170,20 @@ class Bank(object):
         res = self.start_update()
         self.session.set('workflow_status', res)
         self.save_session()
-        self.__stats()
+        try:
+            self.__stats()
+        except Exception:
+            logging.exception('Failed to send stats')
+        end_time = datetime.now()
+        end_time = time.mktime(end_time.timetuple())
+        self.history.insert({
+            'bank': self.name,
+            'error': not res,
+            'start': start_time,
+            'end': end_time,
+            'action': 'update',
+            'updated': self.session.get('update')
+        })
         return res
 
     def __stats(self):
@@ -1139,7 +1214,8 @@ class Bank(object):
             else:
                 influxdb = InfluxDBClient(host=db_host, port=db_port, database=db_name)
         except Exception as e:
-            logging.error('InfluxDB connection error: ' + str(e))
+            logging.error('Failed to connect to InfluxDB, database may not be created, skipping the record of statistics')
+            logging.exception('InfluxDB connection error: ' + str(e))
             return
         metrics = []
 
@@ -1230,7 +1306,14 @@ class Bank(object):
             }
             metrics.append(influx_metric)
 
-        res = influxdb.write_points(metrics, time_precision="s")
+        res = None
+        try:
+            res = influxdb.write_points(metrics, time_precision="s")
+        except Exception as e:
+            logging.error('Failed to connect to InfluxDB, database may not be created, skipping the record of statistics')
+            logging.exception('InfluxDB connection error: ' + str(e))
+            return
+
         if not res:
             logging.error('Failed to send metrics to database')
 
