@@ -11,6 +11,7 @@ import re
 import traceback
 import json
 import hashlib
+import sys
 
 from biomaj_core.utils import Utils
 from biomaj_download.downloadclient import DownloadClient
@@ -682,7 +683,11 @@ class UpdateWorkflow(Workflow):
                 # Download and extract
                 tmp_dir = tempfile.mkdtemp('biomaj')
                 rel_files = release_downloader.download(tmp_dir)
-                rel_file = open(tmp_dir + '/' + rel_files[0]['name'])
+                rel_file = None
+                if (sys.version_info > (3, 0)):
+                    rel_file = open(tmp_dir + '/' + rel_files[0]['name'], encoding='utf-8')
+                else:
+                    rel_file = open(tmp_dir + '/' + rel_files[0]['name'])
                 rel_content = rel_file.read()
                 rel_file.close()
                 shutil.rmtree(tmp_dir)
@@ -1480,13 +1485,43 @@ class UpdateWorkflow(Workflow):
             return True
         no_extract = self.session.config.get('no.extract')
         if no_extract is None or no_extract == 'false':
+            archives = []
             for file in self.downloaded_files:
                 if 'save_as' not in file:
                     file['save_as'] = file['name']
                 nb_try = 1
+                origFile = self.session.get_offline_directory() + '/' + file['save_as']
+                is_archive = False
+                if origFile.endswith('.tar.gz'):
+                    is_archive = True
+                elif origFile.endswith('.tar'):
+                    is_archive = True
+                elif origFile.endswith('.bz2'):
+                    is_archive = True
+                elif origFile.endswith('.gz'):
+                    is_archive = True
+                elif origFile.endswith('.zip'):
+                    is_archive = True
+
+                logging.info('Workflow:wf_uncompress:Uncompress:' + origFile)
+                if not os.path.exists(origFile):
+                    logging.warn('Workflow:wf_uncompress:NotExists:' + origFile)
+                    continue
+
+                tmpCompressedFile = origFile
+                if is_archive:
+                    tmpFileNameElts = file['save_as'].split('/')
+                    tmpFileNameElts[len(tmpFileNameElts) - 1] = 'tmp_' + tmpFileNameElts[len(tmpFileNameElts) - 1]
+                    tmpCompressedFile = self.session.get_offline_directory() + '/' + '/'.join(tmpFileNameElts)
+                    archives.append({'from': origFile, 'to': tmpCompressedFile})
+                else:
+                    continue
+
+                shutil.copy(origFile, tmpCompressedFile)
+
                 not_ok = True
                 while nb_try < 3 and not_ok:
-                    status = Utils.uncompress(self.session.get_offline_directory() + '/' + file['save_as'])
+                    status = Utils.uncompress(origFile)
                     if status:
                         not_ok = False
                     else:
@@ -1494,7 +1529,17 @@ class UpdateWorkflow(Workflow):
                         nb_try += 1
                 if not_ok:
                     logging.error('Workflow:wf_uncompress:Failure:' + file['name'])
+                    # Revert archive files
+                    for archive in archives:
+                        if os.path.exists(archive['to']):
+                            logging.info("Workflow:wf_uncompress:RevertArchive:" + archive['from'])
+                            shutil.move(archive['to'], archive['from'])
                     return False
+            for archive in archives:
+                if os.path.exists(archive['to']):
+                    logging.info("Workflow:wf_uncompress:RemoveAfterExtract:" + archive['to'])
+                    os.remove(archive['to'])
+
         else:
             logging.info("Workflow:wf_uncompress:NoExtract")
         return True
